<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Modules\Cadastros\Services\FilialAlcadasService;
use Modules\Cadastros\Controllers\AprovadorController;
use Modules\Cadastros\Services\AprovadorService;

class FilialAlcadaController
{
    
    public function __construct(
        FilialAlcadasService $FilialAlcadasService,
        AprovadorController $AprovadorController,
        AprovadorService $AprovadorService
    ) {
        $this->FilialAlcadasService = $FilialAlcadasService;
        $this->AprovadorController  = $AprovadorController;
        $this->AprovadorService     = $AprovadorService;
    }
    
    public function saveFilialAlcada(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row                        = $request->getRow();
            $nrConfiguracaoAlcada       = $row['NRCONFIGALCADA'];
            $consistirValorAprovacaoPor = $row['IDCONSISTVRAPROV'];
            $messageParam               = [];
            
            if (isset($row['__is_new']) && $row['__is_new']) {
                $alcada = $this->FilialAlcadasService->findAlcada($nrConfiguracaoAlcada);
                $deveImportarGestoresAutomaticamente = ($alcada['IDIMPGESTFIL'] == 'S' || $alcada['IDTPALCADA'] == 'COMPRA');
                
                foreach ($row['CDFILIAL'] as $codigoFilial) {
                    $this->FilialAlcadasService->saveFilialAlcada($codigoFilial, $nrConfiguracaoAlcada, $consistirValorAprovacaoPor);
                    
                    if ($deveImportarGestoresAutomaticamente) {
                        $messageParam[] = $this->AprovadorController->importaGestoresAuto($codigoFilial, $nrConfiguracaoAlcada);
                    }
                }
                
                if ($deveImportarGestoresAutomaticamente) {
                    self::updateMessagesWithFilialName($row['NMFILIAL'], $messageParam);
                }
                
                $response->addMessage(new Message($messageParam));
            } else {
                $filialAlcada = $this->FilialAlcadasService->findFilialAlcada($row['NRFILIALCADA']);
                
                if ($filialAlcada['APROVACAO'] > 0) {
                    throw new \Exception('OPERATION_BLOCKED_UNIT_PENDING_APPROVALS');
                }
                
                $this->FilialAlcadasService->updateFilialAlcada($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function updateMessagesWithFilialName($nomeFiliais, &$messageParam)
    {
        foreach ($nomeFiliais as $index => $nome) {
            $messageParam[$index] = "Unidade: " . $nome . "|" . $messageParam[$index];
        }
    }
    
    public function deleteFilial(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
        
            $filial = $this->FilialAlcadasService->findFilialAlcada($row['NRFILIALCADA']);
            
            if ($filial['APROVACAO'] > 0) {
                throw new \Exception('OPERATION_BLOCKED_UNIT_PENDING_APPROVALS');
            }
            
            $this->AprovadorService->deleteTodosAprovadoresRelacionadosFilialAlcada($row);
            $this->AprovadorService->deleteTodosAprovadoresFilialAlcada($row);
            $this->FilialAlcadasService->deleteFilialAlcada($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function cadastraFilialAuto(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $responseUnidadesCadastradas = [];
        
            $row = $request->getRow();
            
            foreach ($row['NRCONFIGALCADA'] as $nrAlcada) {
                foreach ($row['IDTPALCADA'] as $idTpAlcada) {
                    $alcada = $this->FilialAlcadasService->findAlcada($nrAlcada);
                    
                    $unidadesCadastradas = [
                        $alcada['NMALCADA']
                    ];
                    
                    foreach ($row['CDFILIAL'] as $filial) {
                        try {
                            $idTpAlcada = 'T';
                            $this->FilialAlcadasService->saveFilialAlcada($filial, $nrAlcada, $idTpAlcada);
                            if ($alcada['IDIMPGESTFIL'] == 'S' || $alcada['IDTPALCADA'] == 'COMPRA') {
                                $this->AprovadorController->importaGestoresAuto($filial, $nrAlcada);
                            }
                        } catch(\Exception $e) {
                            $unidadesCadastradas[] = $filial;
                        }
                    }
                }
                
                if (((is_array($unidadesCadastradas) && !is_null($unidadesCadastradas)) ? count($unidadesCadastradas) : 0) > 1) {
                    $responseUnidadesCadastradas[] = implode('|', $unidadesCadastradas);
                }
            }
            
            if ($responseUnidadesCadastradas) {
                $response->addMessage(new Message($responseUnidadesCadastradas));
                $response->setError(new Error('THERE_ARE_UNITS_ALREADY_REGISTERED', 500));
            }
        
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}
