<?php
namespace Modules\Solicitacao\Modules\Relatorios\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Modules\Solicitacao\Modules\Relatorios\Services\Sup97000VerificacaoCadastrosProcessosComprasSuprimentosService;

class Sup97000VerificacaoCadastrosProcessosComprasSuprimentos extends \Teknisa\GER\Controllers\TeknisaCrud  
{
    
    protected $environment;
    protected $connection;
    protected $dataSourceManager;
    protected $dataSourceName = "sup08200_relatorio";
    
   
    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $connection, $service) 
    {
        $this->crudService       = $crudService;
        $this->parameterBag =   $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->service           = $service;
    }
    
    public function regiao (DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getRow();

        $regiao = $this->service->regiaoFilial($row);
        
        $response->addDataSet(new DataSet("resp", $regiao));
    }
    
    public function verificaProduto (DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getRow();

        $produto = $this->service->verificaProdutoAtivo($row);
        
        $response->addDataSet(new DataSet("resp", $produto));
    }
    
    public function verificaCalculoNecessidadeCompras (DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getRow()[0];
        
        $FilialSemRegiao             = $this->service->FilialSemRegiao($row);
        $FilialSemContratoReincidido = $this->service->FilialSemContratoReincidido($row);
        $ProdutoPorFilial            = $this->service->ProdutoPorFilial($row);
        $DadosProdutoFilial          = $this->service->DadosProdutoFilial($row);
        
        if (empty($FilialSemRegiao)) {
            $erro = "1";
        } elseif (empty($FilialSemContratoReincidido)) {
            $erro = "2";
        } elseif (empty($ProdutoPorFilial)) {
            $erro = "3";
        } elseif (empty($DadosProdutoFilial)) {
            $erro = "4";
        } else {
            $CalendarioDeCompras = $this->service->CalendarioDeCompras($row, $DadosProdutoFilial);
            if (empty($CalendarioDeCompras)) {
                $erro = "5";
            } else {
                $erro = "0";
            }
        }
        
        $response->addDataSet(new DataSet("resp", array($erro)));
    }
    
    public function verificarLancamentoSoliComp (DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getRow()[0];
        
        $FilialSemRegiao             = $this->service->FilialSemRegiao($row);
        $FilialSemContratoReincidido = $this->service->FilialSemContratoReincidido($row);
        $ProdutoPorFilial            = $this->service->ProdutoPorFilial($row);
     
        if (empty($FilialSemRegiao)) { //abre tela Regiao
            $erro = "1";
        } elseif (empty($FilialSemContratoReincidido)) { //filial
            $erro = "2";
        } elseif (empty($ProdutoPorFilial)) {
            $erro = "3";
        } else {
            $erro = "0";
        }
        
        $response->addDataSet(new DataSet("resp", array($erro)));
    }
    
    public function verificarPlanilhaCotacao (DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getRow()[0];
        $ConsultaParametro  = $this->service->ConsultaParametro($row);
      
        if (empty($ConsultaParametro)) {
            $result = "S";
        } else {
            $ProdutoPorFilial                   = $this->service->ProdutoPorFilial($row);
            $ConsultaMarcaPorProduto            = $this->service->ConsultaMarcaPorProduto($row);
            $ConsultaQualificacaoMarca          = $this->service->ConsultaQualificacaoMarca($row, $ConsultaParametro);
            $ConsultaProdutoPorFornecedor       = $this->service->ConsultaProdutoPorFornecedor($row);
            $ConsultaQualificacaoFornecedor     = $this->service->ConsultaQualificacaoFornecedor($row, $ConsultaParametro);
            $ConsultaQualificacaoProdFornecedor = $this->service->ConsultaQualificacaoProdFornecedor($row, $ConsultaParametro); 
            $ConsultaFornecedorNaoAtendeRegiao  = $this->service->ConsultaFornecedorNaoAtendeRegiao($row, $ConsultaParametro);
            $ConsultaFornecedorNaoAtendeFilial  = $this->service->ConsultaFornecedorNaoAtendeFilial($row);
            
            $ConsultaProdutoPorComprador        = $this->service->ConsultaProdutoPorComprador($row);

            if ($ConsultaMarcaPorProduto[0]['MARCAPROD'] == 0) {
                $result = "1"; //abrir tela marca por produto
            } elseif ($ConsultaFornecedorNaoAtendeFilial[0]['FORNFIL'] != 0) {
                 $result = "2"; //abre tela Região não atendida pelo Fornecedor
            } elseif ($ConsultaQualificacaoMarca[0]['QLFMARCA'] == 0) {
                $result = "3"; //abre tela Qualificação de Marca por Produto*
            } elseif ($ConsultaProdutoPorFornecedor[0]['PRODFORN'] == 0) {
                $result = "4"; //abre tela Produto por Fornecedor ou Fornecedor por Produto
            } elseif ($ConsultaQualificacaoFornecedor[0]['QLFFORN'] == 0) {
                $result = "5"; //abre tela Qualifição de Fornecedor
            } elseif ($ConsultaQualificacaoProdFornecedor[0]['QLFPRODFORN'] == 0) {
                $result = "6"; //abre tela Qualificação de Produto por Fornecedor
            } elseif ($ConsultaFornecedorNaoAtendeRegiao[0]['FORNREG'] == 0) {
                 $result = "7"; //abre tela Região não atendida pelo Fornecedor
            } elseif (empty($ProdutoPorFilial)) {
                $result = "8"; //abre Tela Produtos por Filial 
            } elseif (empty($ConsultaProdutoPorComprador)) {
                $result = "9"; //abre Tela de Comprador
            } else {
                $result = "0";
            }
        }
        $response->addDataSet(new DataSet("resp", array($result)));
    }
    
    public function verificarApuracaoEfetivacao (DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getRow()[0];
        $ConsultaParametro = $this->service->ConsultaParametro($row);
        
        if (empty($ConsultaParametro)) {
            $erro = "S";
        } else {
            $FilialSemContratoReincidido        = $this->service->FilialSemContratoReincidido($row);
            $ProdutoPorFilial                   = $this->service->ProdutoPorFilial($row);
            $ConsultaRegiao                     = $this->service->ConsultaRegiao($row);
            $ConsultaMarcaPorProduto            = $this->service->ConsultaMarcaPorProduto($row);
            $ConsultaQualificacaoMarca          = $this->service->ConsultaQualificacaoMarca($row, $ConsultaParametro);
            $ConsultaProdutoPorFornecedor       = $this->service->ConsultaProdutoPorFornecedor($row);
            $ConsultaQualificacaoFornecedor     = $this->service->ConsultaQualificacaoFornecedor($row, $ConsultaParametro);
            $ConsultaQualificacaoProdFornecedor = $this->service->ConsultaQualificacaoProdFornecedor($row, $ConsultaParametro);
            $ConsultaFornecedorNaoAtendeRegiao  = $this->service->ConsultaFornecedorNaoAtendeRegiao($row, $ConsultaParametro);
            $ConsultaFornecedorNaoAtendeFilial  = $this->service->ConsultaFornecedorNaoAtendeFilial($row);
            $ConsultaProdPreferencial           = $this->service->ConsultaProdPreferencial($row);
            
            $ConsultaProdutoPorComprador        = $this->service->ConsultaProdutoPorComprador($row);
          
            if (empty($FilialSemContratoReincidido)) {
                $erro = "1"; // Filial 
            } 
            elseif (empty($ProdutoPorFilial)) {
                $erro = "2"; // abrir Tela Produtos por Filial 
            }
            elseif ($ConsultaRegiao[0]['REG'] == 0) {
                $erro = "3"; // abrir Tela Região
            } elseif ($ConsultaMarcaPorProduto[0]['MARCAPROD'] == 0) {
                $erro = "4";//abrir tela marca por produto
            } elseif ($ConsultaFornecedorNaoAtendeFilial[0]['FORNFIL'] != 0) {
                $erro = "5"; 
            } elseif ($ConsultaQualificacaoMarca[0]['QLFMARCA'] == 0) {
                $erro = "6";//abre tela Qualificação de Marca por Produto
            } elseif ($ConsultaProdutoPorFornecedor[0]['PRODFORN'] == 0) {
                $erro = "7"; //abre tela Produto por Fornecedor ou Fornecedor por Produto
            } elseif ($ConsultaQualificacaoFornecedor[0]['QLFFORN'] == 0) {
                $erro = "8"; //abre tela Qualifição de Fornecedor
            } elseif ($ConsultaQualificacaoProdFornecedor[0]['QLFPRODFORN'] == 0) {
                $erro = "9"; //abre tela Qualificação de Produto por Fornecedor
            } elseif ($ConsultaFornecedorNaoAtendeRegiao[0]['FORNREG'] == 0) {
                $erro = "10"; // abre tela Região não atendida pelo Fornecedor
            } elseif ($ConsultaProdPreferencial[0]['PRODPREF'] > 0) {
                $ConsultaQtdMedia = $this->service->ConsultaQtdMedia($row);
                
                if ($ConsultaQtdMedia[0]['QTDMEDIA'] == 0) {
                    $erro = "11"; //abre tela Produto Preferencial/Quantidade Média Solicitada
                } elseif ($ConsultaParametro[0]['IDUTISTATCOT'] == 'S') {
                    $ConsultaIdutistatcot = $this->service->ConsultaIdutistatcot($row, $ConsultaParametro);
                    
                    if ($ConsultaIdutistatcot == 0) {
                        $erro = "12"; //abre Tela Planilha de Cotação
                    } else {
                        $erro = "0"; //Verificação concluída!
                    }
                }
            } elseif (!empty($ConsultaProdutoPorComprador)) {
                 $erro = "13"; //abre Tela de Comprador
            } else {
                 $erro = "0"; //Verificação concluída!
            }
        }
        
        $response->addDataSet(new DataSet("resp", array($erro)));
    }
    
    public function verificarLancamentoAf (DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getRow()[0];
        
        $ConsultaParametro = $this->service->ConsultaParametro($row);
        
        if (empty($ConsultaParametro)) {
            $erro = "S";
        } else {
            $ConsultaRegiao                     = $this->service->ConsultaRegiao($row);
            $FilialSemContratoReincidido        = $this->service->FilialSemContratoReincidido($row);
            // $ProdutoPorFilial                = $this->service->ProdutoPorFilial($row);
            $ConsultaMarcaPorProduto            = $this->service->ConsultaMarcaPorProduto($row);
            $ConsultaQualificacaoMarca          = $this->service->ConsultaQualificacaoMarca($row, $ConsultaParametro);
            $ConsultaProdutoPorFornecedor       = $this->service->ConsultaProdutoPorFornecedor($row);
            $ConsultaQualificacaoFornecedor     = $this->service->ConsultaQualificacaoFornecedor($row, $ConsultaParametro);
            $ConsultaQualificacaoProdFornecedor = $this->service->ConsultaQualificacaoProdFornecedor($row, $ConsultaParametro);
            $ConsultaFornecedorNaoAtendeRegiao  = $this->service->ConsultaFornecedorNaoAtendeRegiao($row, $ConsultaParametro);
            $ConsultaFornecedorNaoAtendeFilial  = $this->service->ConsultaFornecedorNaoAtendeFilial($row);
            $ConsultaProdPreferencial           = $this->service->ConsultaProdPreferencial($row);
            
            $ConsultaProdutoPorComprador        = $this->service->ConsultaProdutoPorComprador($row);

            if($ConsultaRegiao[0]['REG'] == 0) {
                $erro = "1"; // abrir Tela Região
            } elseif (empty($FilialSemContratoReincidido)) {
                $erro = "2"; //abrir tela Filial 
            } 
            // else if(empty($ProdutoPorFilial)) {
            //     $erro = "3"; // abrir Tela Produtos por Filial 
            // } 
            elseif ($ConsultaMarcaPorProduto[0]['MARCAPROD'] == 0) {
                $erro = "4";//abrir tela marca por produto
            } elseif ($ConsultaFornecedorNaoAtendeFilial[0]['FORNFIL'] != 0) {
                $erro = "5"; 
            } elseif ($ConsultaQualificacaoMarca[0]['QLFMARCA'] == 0) {
                $erro = "6";//abre tela Qualificação de Marca por Produto
            } elseif ($ConsultaProdutoPorFornecedor[0]['PRODFORN'] == 0) {
                $erro = "7"; //abre tela Produto por Fornecedor ou Fornecedor por Produto
            } elseif ($ConsultaQualificacaoFornecedor[0]['QLFFORN'] == 0) {
                $erro = "8"; //abre tela Qualifição de Fornecedor
            } elseif ($ConsultaQualificacaoProdFornecedor[0]['QLFPRODFORN'] == 0) {
                $erro = "9"; //abre tela Qualificação de Produto por Fornecedor
            } elseif ($ConsultaFornecedorNaoAtendeRegiao[0]['FORNREG'] == 0) {
                $erro = "10"; // abre tela Região não atendida pelo Fornecedor
            } elseif ($ConsultaProdPreferencial[0]['PRODPREF'] > 0) {
                $ConsultaQtdMedia = $this->service->ConsultaQtdMedia($row);
                
                if ($ConsultaQtdMedia[0]['QTDMEDIA'] == 0) {
                    $erro = "11"; //abre tela Produto Preferencial/Quantidade Média Solicitada
                } else {
                    $erro = "0"; //Verificação concluída!
                }
            } elseif (empty($ConsultaProdutoPorComprador)) {
                $erro = "12"; //abre Tela de Comprador
            } else {
                 $erro = "0"; //Verificação concluída!
            }
        }
        
        $response->addDataSet(new DataSet("resp", array($erro)));
    }
    
    public function verificarCustoLista (DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getRow()[0];

        $regiao     = $this->service->regiaoFilial($row);
        $resultado  = $this->service->VerificarCustoLista($row, $regiao);
      
        if (empty($resultado)) {
            $resp = "N";
        } else {
            $concatResult = 
                $resultado[0]['IDPRODPREF'].$resultado[0]['IDREFORFIL'].$resultado[0]['IDREFOREG'].
                $resultado[0]['IDQFOPRODFORN'].$resultado[0]['IDQFOMARPROD'].$resultado[0]['IDQFOPARASUPRI'].
                $resultado[0]['IDFORNECEDOR'].$resultado[0]['IDRESTROMARCA'].$resultado[0]['IDRESTRFORN'].
                $resultado[0]['IDRESTRMARCFORN'].$resultado[0]['IDSITUPART'];
                
            if ($concatResult == '11111111111') {
               $resp = 'OK';
            } else {
                $operadorLista = $this->service->getOperadorLista($row);
                $row['CDOPERADOR'] = !empty($operadorLista) ? $operadorLista[0]['CDOPERADOR'] : "T";
                $ConsultaProdutoPorComprador = $this->service->ConsultaProdutoPorComprador($row);
                
                if ($resultado[0]['IDPRODPREF'] != 1) {
                    $resp = '1';
                } elseif ($resultado[0]['IDREFORFIL'] != 1) {
                    $resp = '2';
                } elseif ($resultado[0]['IDREFOREG'] != 1) {
                    $resp = '3';
                } elseif ($resultado[0]['IDQFOPRODFORN'] != 1) {
                    $resp = '4';
                } elseif ($resultado[0]['IDQFOMARPROD'] != 1) {
                    $resp = '5';
                } elseif ($resultado[0]['IDQFOPARASUPRI'] != 1) {
                    $resp = '6';
                } elseif ($resultado[0]['IDFORNECEDOR'] != 1) {
                    $resp = '7';
                } elseif ($resultado[0]['IDRESTROMARCA'] != 1) {
                    $resp = '8';
                } elseif ($resultado[0]['IDRESTRFORN'] != 1) {
                    $resp = '9';
                } elseif ($resultado[0]['IDRESTRMARCFORN'] != 1) {
                    $resp = '10';
                } elseif ($resultado[0]['IDSITUPART'] != 1) {
                    $resp = '11';
                } elseif (empty($ConsultaProdutoPorComprador)) {
                    $resp = "12"; //abre Tela de Comprador
                }
            }
        }
            
        $response->addDataSet(new DataSet("resp", [$resp]));
    }
    
    public function GridRelatorio(Request\Filter $request, Response $response) 
    {
        $onDemandFilter    = $request->getFilterCriteria();
        $conditions        = $onDemandFilter->getConditions();
        $onDemandFilter->setDataSourceName($this->dataSourceName);
        $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($onDemandFilter, array());
        $dataSource        = $this->dataSourceManager->findBy($newFilterCriteria);
        $dataSet           = $dataSource->getRows();
    
        if (empty($dataSet)) {
            $msg = 'Produto informado não está na lista ou não possui cotação.';
            $response->addDataSet(new DataSet('sup08200_relatorio', $dataSet));
        } else {
            $status = explode('-', $dataSet[0]['IDSTATUS']);
            
            if ($status[0] == 1) {
                $status[0] = 'OK';
                //$response->addMessage(new Message('Operação realizada com sucesso!'));
            } else {
                $status[0] = 'Produto Preferencial';
            }
            
            if ($status[1] == 1) {
                $status[1] = '';
            } else {
                $status[1] = '_Fornecedor não Atende a Unidade' ; 
            }
            
            if ($status[2] == 1) {
                $status[2] = '';
            } else {
                $status[2] = '_Fornecedor não Atende a Região' ; 
            }
            
            if ($status[3] == 1) {
                $status[3] = '';
            } else {
                $status[3] = '_Qualificação de Produto por Fornecedor' ; 
            }
            
            if ($status[4] == 1) {
                $status[4] = '';
            } else {
                $status[4] = '_Qualificação da Marca' ; 
            }
            
            if ($status[5] == 1) {
                $status[5] = '';
            } else {
                $status[5] = '_Qualificação da Marca' ; 
            }
            
            if ($status[6] == 1) {
                $status[6] = '';
            } else {
                $status[6] = '_Cadastro de Fornecedor (alternativo, inativo e/ou desqualificado)' ; 
            }
            
            if ($status[7] == 1) {
                $status[7] = '';
            } else {
                $status[7] = '_Restrição à Marca' ; 
            }
            
            if ($status[8] == 1) {
                $status[8] = '';
            } else {
                $status[8] = '_Restrição ao Fornecedor' ; 
            }
            
            if ($status[9] == 1) {
                $status[9] = '';
            } else {
                $status[9] = '_Restrição à marca/fornecedor' ; 
            }
            
            if ($status[10] == 1) {
                $status[10] = '';
            } else {
                $status[10] = '_Particularidade do Produto' ; 
            }
            
            for ($i = 0, $j = count($dataSet); $i<$j; $i++) {
                $dataSet[$i]['IDSTATUS'] = str_replace('-', '', (join('-', $status)));
            }
           
            $response->addDataSet(new DataSet('sup08200_relatorio', $dataSet));
        }
    }     
}
